#==============================================================================
# ■ ScriptExtracter
#------------------------------------------------------------------------------
# Auther Tashiraka
#==============================================================================

module ScriptManager
  SCRIPTS_FILE  = "Scripts.rvdata2"
  INPUT         = "_Input"
  OUTPUT        = "_Output"
  ROOT          = "Scripts"
  PRESET        = "1_Preset"
  PLUGINS       = "2_Plugins"
  HOMEMADE      = "3_Homemade"
  TEMPORARY     = "0_Temporary"
  DATA          = "1_Data"
  MANAGERS      = "2_Managers"
  OBJECTS       = "3_Objects"
  SPRITES       = "4_Sprites"
  WINDOWS       = "5_Windows"
  SCENES        = "6_Scenes"

  EXTRACT_PATH_LIST = [
    # 指定行数のセクションを指定のフォルダに流し込む設定です。
    # Scripts.rvdata2のセクションの行数にrange: (x..x)が対応しています。(0行から始まります)
    # 基本的にプリセット部分を弄る必要はありません。
    # プラグインとホームメイドの行数を適当に調整してください。
    # 使用しない場合は適当にrange範囲を上げておけばおｋです。
    # プリセット
    {range: (2..7), path: "#{OUTPUT}/#{ROOT}/#{PRESET}/#{MANAGERS}"},
    {range: (10..40), path: "#{OUTPUT}/#{ROOT}/#{PRESET}/#{OBJECTS}"},
    {range: (43..50), path: "#{OUTPUT}/#{ROOT}/#{PRESET}/#{SPRITES}"},
    {range: (53..97), path: "#{OUTPUT}/#{ROOT}/#{PRESET}/#{WINDOWS}"},
    {range: (100..118), path: "#{OUTPUT}/#{ROOT}/#{PRESET}/#{SCENES}"},
    # プラグイン
    {range: (121..121), path: "#{OUTPUT}/#{ROOT}/#{PLUGINS}"},
    # ホームメイド
    {range: (1000..1000), path: "#{OUTPUT}/#{ROOT}/#{HOMEMADE}/#{TEMPORARY}"},
    {range: (1001..1001), path: "#{OUTPUT}/#{ROOT}/#{HOMEMADE}/#{DATA}"},
    {range: (1002..1002), path: "#{OUTPUT}/#{ROOT}/#{HOMEMADE}/#{MANAGERS}"},
    {range: (1003..1003), path: "#{OUTPUT}/#{ROOT}/#{HOMEMADE}/#{OBJECTS}"},
    {range: (1004..1004), path: "#{OUTPUT}/#{ROOT}/#{HOMEMADE}/#{SPRITES}"},
    {range: (1005..1005), path: "#{OUTPUT}/#{ROOT}/#{HOMEMADE}/#{WINDOWS}"},
    {range: (1006..1006), path: "#{OUTPUT}/#{ROOT}/#{HOMEMADE}/#{SCENES}"},
  ]

  STORE_DELIMITER_LIST = {
    "#{INPUT}/#{ROOT}/#{PRESET}/#{MANAGERS}" =>    "▼モジュール",
    "#{INPUT}/#{ROOT}/#{PRESET}/#{OBJECTS}" =>     "▼オブジェクト",
    "#{INPUT}/#{ROOT}/#{PRESET}/#{SPRITES}" =>     "▼スプライト",
    "#{INPUT}/#{ROOT}/#{PRESET}/#{WINDOWS}" =>     "▼ウィンドウ",
    "#{INPUT}/#{ROOT}/#{PRESET}/#{SCENES}" =>      "▼シーン",
    "#{INPUT}/#{ROOT}/#{PLUGINS}" =>               "▼素材",
    "#{INPUT}/#{ROOT}/#{HOMEMADE}/#{TEMPORARY}" => "▽一時設定",
    "#{INPUT}/#{ROOT}/#{HOMEMADE}/#{DATA}" =>      "▽データ",
    "#{INPUT}/#{ROOT}/#{HOMEMADE}/#{MANAGERS}" =>  "▽モジュール",
    "#{INPUT}/#{ROOT}/#{HOMEMADE}/#{OBJECTS}" =>   "▽オブジェクト",
    "#{INPUT}/#{ROOT}/#{HOMEMADE}/#{SPRITES}" =>   "▽スプライト",
    "#{INPUT}/#{ROOT}/#{HOMEMADE}/#{WINDOWS}" =>   "▽ウィンドウ",
    "#{INPUT}/#{ROOT}/#{HOMEMADE}/#{SCENES}" =>    "▽シーン",
  }
  #------------------------------------------------------------------------------
  # ● メンバ変数の初期化
  #------------------------------------------------------------------------------
  def self.init_members
    @data_scripts = []
  end
  #------------------------------------------------------------------------------
  # ● マジックナンバーの取得
  #------------------------------------------------------------------------------
  def self.magic_number
    Random.rand(100000000)
  end
  #------------------------------------------------------------------------------
  # ● Scripts.rvdata2の展開
  #------------------------------------------------------------------------------
  def self.extract_scripts
    init_members
    puts "展開作業を始めるゾ"
    unless load_scripts
      puts "#{SCRIPTS_FILE}の読み込みに失敗したゾ"
      return
    end
    make_extract_directory
    EXTRACT_PATH_LIST.each{|range_hash|
      range_hash[:range].each_with_index{|section_index, i|
        dump_extract_file(range_hash[:path], section_index, i)
      }
    }
    make_development_scripts_file

    puts "展開完了したゾ"
  end
  #------------------------------------------------------------------------------
  # ● .rbの圧縮
  #------------------------------------------------------------------------------
  def self.store_scripts
    init_members
    puts "圧縮作業を始めるゾ"

    prev_dirname = ""
    store_file_path_list("#{INPUT}/#{ROOT}").each{|file_path|
      current_dirname = File.dirname(file_path)
      if prev_dirname != current_dirname
        prev_dirname = current_dirname
        store_push_script("", "")
        store_push_script(STORE_DELIMITER_LIST[current_dirname], "")
      end
      section_name = ""
      section_name = $1 if file_path =~ /\d{4}\_(\S+)\.rb$/
      File.open(file_path, "rb"){|f| store_push_script(section_name, f.read)}
      # File.open(file_path, "rb:utf-8"){|f| store_push_script(section_name, f.read)}
    }
    store_push_script("", "")
    store_push_script("▼メイン", "")
    store_push_script("Main",  RELEASE_MAIN_SECTION)
    Dir.mkdir("#{OUTPUT}/Data")
    dump_store_scripts("#{OUTPUT}/Data/#{SCRIPTS_FILE}")

    puts "圧縮完了したゾ"
  end
  #------------------------------------------------------------------------------
  # ● スクリプトファイルの読み込み
  #------------------------------------------------------------------------------
  def self.load_scripts
    load_scripts_without_rescue rescue false
  end
  #------------------------------------------------------------------------------
  # ● スクリプトファイルの読み込み（例外処理無し）
  #------------------------------------------------------------------------------
  def self.load_scripts_without_rescue
    scripts_path = "#{INPUT}/#{SCRIPTS_FILE}"
    File.open(scripts_path, "rb") do |file|
      @data_scripts = Marshal.load(file)
    end
    return true
  end
  #------------------------------------------------------------------------------
  # ● 展開ディレクトリの作成
  #------------------------------------------------------------------------------
  def self.make_extract_directory
    path_list = extract_directory_list
    path_list.each{|path| Dir.mkdir(path)}
    puts "展開ディレクトリ作成したゾ"
  end
  #------------------------------------------------------------------------------
  # ● 展開ディレクトリリストの取得
  #------------------------------------------------------------------------------
  def self.extract_directory_list
    d1 = "#{OUTPUT}/#{ROOT}"
    d2_list = [PLUGINS, PRESET, HOMEMADE]
    d3_list = [
      TEMPORARY,
      DATA,
      MANAGERS,
      OBJECTS,
      SPRITES,
      WINDOWS,
      SCENES,
    ]
    list = [d1]
    d2_list.each{|d2|
      list.push "#{d1}/#{d2}"
      next if d2 == PLUGINS
      d3_list.each{|d3|
        next if d2 == PRESET && (d3 == TEMPORARY || d3 == DATA)
        list.push "#{d1}/#{d2}/#{d3}"
      }
    }
    return list
  end
  #------------------------------------------------------------------------------
  # ● スクリプトデータの出力
  #------------------------------------------------------------------------------
  def self.dump_store_scripts(path)
    begin
      save_scripts_without_rescue(path)
    rescue
#      FileUtils.remove(path)
      File.delete(path)
      false
    end
  end
  #------------------------------------------------------------------------------
  # ● Scripts.rvdata2の出力（例外処理なし）
  #------------------------------------------------------------------------------
  def self.save_scripts_without_rescue(path)
    File.open(path, "wb") do |file|
      Marshal.dump(@data_scripts, file)
    end
    return true
  end
  #------------------------------------------------------------------------------
  # ● 展開ファイルの出力
  #------------------------------------------------------------------------------
  def self.dump_extract_file(directory, section_index, i)
    return false if @data_scripts[section_index].nil? ||
      @data_scripts[section_index][1] == "" ||
      @data_scripts[section_index][2].nil?

    file_name = sprintf("%04d_%s.rb", (i+1)*10, @data_scripts[section_index][1])
    file_path = "#{directory}/#{file_name}"
    file_buffer = Zlib::Inflate.inflate(@data_scripts[section_index][2])
    File.open(file_path, "wb") do |file|
      # Marshal.dump(file_buffer, file)
      file.write file_buffer
    end
    puts "#{file_path}出力したゾ"
    return true
  end
  #------------------------------------------------------------------------------
  # ● 開発用スクリプトファイルの作成
  #------------------------------------------------------------------------------
  def self.make_development_scripts_file
    @data_scripts = []
    main_section = []
    main_section[0] = magic_number
    main_section[1] = "開発用メインセクション"
    main_section[2] = Zlib::Deflate.deflate(DEVELOPMENT_MAIN_SECTION)
    @data_scripts.push main_section
    Dir.mkdir("#{OUTPUT}/Data")
    dump_store_scripts("#{OUTPUT}/Data/#{SCRIPTS_FILE}")
    puts "#{SCRIPTS_FILE}を出力したゾ"
  end
  #------------------------------------------------------------------------------
  # ● 圧縮用ファイルパスを取得
  #------------------------------------------------------------------------------
  def self.store_file_path_list(path)
    list = []
    Dir.foreach(path, encoding: "utf-8"){|name|
      next if [".", ".."].include?(name)
      path_name = sprintf("%s/%s", path, name)
      list.push(path_name) if name =~ /.rb$/
      list.concat(store_file_path_list(path_name)) if Dir.exist?(path_name)
    }
    return list
  end
  #------------------------------------------------------------------------------
  # ● 圧縮リストに追加
  #------------------------------------------------------------------------------
  def self.store_push_script(section_name, script)
    section = []
    section[0] = magic_number
    section[1] = section_name
    section[2] = Zlib::Deflate.deflate(script)
    @data_scripts.push(section)
  end
end

ScriptManager::DEVELOPMENT_MAIN_SECTION = <<-EOS
#==============================================================================
# ■ Main
#------------------------------------------------------------------------------
# 開発用メインセクション
#==============================================================================
def require_decrypt_files(path)
  Dir.foreach(path){|name|
    next if [".", ".."].include?(name)
    path_name = sprintf("%s/%s", path, name)
    _require path_name if name =~ /.rb$/
    require_decrypt_files(path_name) if Dir.exist?(path_name)
  }
end

def _require(file_name)
  File.open(file_name, "r:utf-8"){|f| eval(f.read)}
end

require_decrypt_files("Scripts")
rgss_main { SceneManager.run }

EOS

ScriptManager::RELEASE_MAIN_SECTION = <<-EOS
#==============================================================================
# ■ Main
#------------------------------------------------------------------------------
# リリース用メインセクション
#==============================================================================
rgss_main { SceneManager.run }

EOS
